﻿using System;
using System.Linq;
using System.Threading;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Core.Networks;
using HeuristicLab.Data;
using HeuristicLab.Encodings.BinaryVectorEncoding;
using HeuristicLab.Problems.DataAnalysis;

namespace HeuristicLab.Networks.FeatureSelection_Network {
  [Item("FeatureSelectionConnector", "")]
  public sealed class FeatureSelectionConnectorCode : Node {
    private FeatureSelectionConnectorCode(FeatureSelectionConnectorCode original, Cloner cloner) : base(original, cloner) { }
    public FeatureSelectionConnectorCode()
      : base() {
      if (Ports.Count == 0)
        Initialize();
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new FeatureSelectionConnectorCode(this, cloner);
    }

    public void Initialize() {
      var parameters = new MessagePort("Parameters");
      Ports.Add(parameters);
      parameters.Parameters.Add(new PortParameter<IRegressionProblemData>("ProblemData") { Type = PortParameterType.Input });

      var selectionPort = new MessagePort("Selection Connector");
      Ports.Add(selectionPort);
      selectionPort.Parameters.Add(new PortParameter<BinaryVector>("Selection") { Type = PortParameterType.Input });
      selectionPort.Parameters.Add(new PortParameter<DoubleValue>("Quality") { Type = PortParameterType.Input | PortParameterType.Output });

      var regressionPort = new MessagePort("Regression Connector");
      Ports.Add(regressionPort);
      regressionPort.Parameters.Add(new PortParameter<IRegressionProblemData>("ProblemData") { Type = PortParameterType.Output });
      regressionPort.Parameters.Add(new PortParameter<IRegressionSolution>("Linear regression solution") { Type = PortParameterType.Input });
      RegisterEvents();
    }

    public void RegisterEvents() {
      var selection = (IMessagePort)Ports["Selection Connector"];
      selection.MessageReceived += Selection_MessageReceived;
    }
    public void DeregisterEvents() {
      var selection = (IMessagePort)Ports["Selection Connector"];
      selection.MessageReceived -= Selection_MessageReceived;
    }

    private void Selection_MessageReceived(object sender, EventArgs<IMessage, CancellationToken> e) {
      // get parameters
      var parametersPort = (IMessagePort)Ports["Parameters"];
      var parameters = parametersPort.PrepareMessage();
      parametersPort.SendMessage(parameters, e.Value2);
      var problemData = (IRegressionProblemData)parameters["ProblemData"];
      problemData = (IRegressionProblemData)problemData.Clone();

      // build coordinates
      var selectionMsg = e.Value;
      var selection = (BinaryVector)selectionMsg["Selection"];
      var allowedVariables = problemData.InputVariables;
      foreach (var t in selection.Zip(allowedVariables, Tuple.Create)) {
        problemData.InputVariables.SetItemCheckedState(t.Item2, t.Item1);
      }

      // solve Regression
      var regressionConPort = (IMessagePort)Ports["Regression Connector"];
      var regressionMsg = regressionConPort.PrepareMessage();
      regressionMsg["ProblemData"] = problemData;
      regressionConPort.SendMessage(regressionMsg, e.Value2);
      var solution = (IRegressionSolution)regressionMsg["Linear regression solution"];

      selectionMsg["Quality"] = new DoubleValue(solution.TestNormalizedMeanSquaredError);
    }
  }
}
