﻿using System;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Encodings.BinaryVectorEncoding;
using HeuristicLab.Optimization;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Networks.FeatureSelection_Network {
  public class SelectionProblem : SingleObjectiveBasicProblem<BinaryVectorEncoding> {
    private Func<BinaryVector, double> eval;

    public virtual int Length {
      get { return Encoding.Length; }
      set { Encoding.Length = value; }
    }

    protected SelectionProblem(SelectionProblem original, Cloner cloner)
      : base(original, cloner) {
    }
    [StorableConstructor]
    protected SelectionProblem(bool deserializing) : base(deserializing) { }

    public SelectionProblem(Func<BinaryVector, double> eval, bool maximization)
      : base() {
      this.maximization = maximization;
      this.eval = eval;


      var lengthParam = new ValueParameter<IntValue>("Length");
      lengthParam.ValueChanged += (o, s) => { Length = lengthParam.Value.Value; };
      Parameters.Add(lengthParam);
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new SelectionProblem(this, cloner);
    }

    public override double Evaluate(Individual individual, IRandom random) {
      return eval(individual.BinaryVector());
    }

    private readonly bool maximization;
    public override bool Maximization {
      get { return maximization; }
    }
  }
}
